<?php
namespace mundophpbb\custom404\acp;

/**
 * Módulo ACP para gerenciar configurações e logs do Custom 404
 */
class main_module
{
    /** @var string */
    public $u_action;

    /** @var string */
    public $tpl_name = '@mundophpbb_custom404/acp_custom404';

    /** @var string */
    public $page_title = 'ACP_CUSTOM404_TITLE';

    /** @var \phpbb\user */
    protected $user;

    /** @var \phpbb\language\language */
    protected $language;

    /** @var \phpbb\filesystem\filesystem */
    protected $filesystem;

    /** @var \phpbb\di\container */
    protected $phpbb_container;

    /**
     * Método principal do módulo ACP
     *
     * @param string $id ID do módulo
     * @param string $mode Modo do módulo
     */
    public function main($id, $mode)
    {
        global $phpbb_container;

        try {
            $this->phpbb_container = $phpbb_container;
            $this->filesystem = $phpbb_container->get('filesystem');
            $this->user = $phpbb_container->get('user');
            $this->language = $phpbb_container->get('language');
            $request = $phpbb_container->get('request');
            $config = $phpbb_container->get('config');
            $template = $phpbb_container->get('template');
            $upload_handler = $phpbb_container->get('mundophpbb.custom404.upload');
            $db = $phpbb_container->get('dbal.conn');

            // Inicializar logging de erros
            $log_dir = $this->phpbb_container->getParameter('core.root_path') . 'ext/mundophpbb/custom404/logs/';
            $log_file = $log_dir . 'debug.log';
            if ($this->filesystem && !$this->filesystem->exists($log_dir)) {
                $this->filesystem->mkdir($log_dir, 0755);
            }
            ini_set('log_errors', 1);
            ini_set('error_log', $log_file);
            error_reporting(E_ALL & ~E_NOTICE);

            $this->load_language_files();
            add_form_key('mundophpbb_custom404');

            $upload_dir = $this->phpbb_container->getParameter('core.root_path') . 'ext/mundophpbb/custom404/uploads/';
            $allowed_extensions = ['png', 'jpg', 'jpeg', 'gif'];
            $max_file_size = 2 * 1024 * 1024; // 2 MB

            if ($this->filesystem && !$this->filesystem->exists($upload_dir)) {
                $this->filesystem->mkdir($upload_dir, 0755);
            }

            $success_message = '';
            $error_message = '';

            // Manipular exclusão de imagem
            if ($request->is_set_post('delete_image')) {
                if (!check_form_key('mundophpbb_custom404')) {
                    $error_message = $this->language->lang('FORM_INVALID');
                } else {
                    $image_to_delete = $request->variable('delete_image_path', '');
                    if ($image_to_delete && $image_to_delete !== 'ext/mundophpbb/custom404/uploads/404_error.png') {
                        $image_full_path = $this->filesystem ? $this->filesystem->realpath($upload_dir . basename($image_to_delete)) : null;
                        if ($image_full_path && $this->filesystem && $this->filesystem->exists($image_full_path)) {
                            $this->filesystem->remove($image_full_path);
                            if ($config['custom404_image_path'] === $image_to_delete) {
                                $config->set('custom404_image_path', 'ext/mundophpbb/custom404/uploads/404_error.png');
                            }
                            $success_message = $this->language->lang('ACP_CUSTOM404_IMAGE_DELETED');
                        } else {
                            $error_message = $this->language->lang('ACP_CUSTOM404_IMAGE_NOT_FOUND');
                        }
                    } else {
                        $error_message = $this->language->lang('ACP_CUSTOM404_INVALID_IMAGE');
                    }
                }
            }

            // Manipular upload de imagem
            if ($request->is_set_post('upload')) {
                if (!check_form_key('mundophpbb_custom404')) {
                    $error_message = $this->language->lang('FORM_INVALID');
                } else {
                    try {
                        $uploaded_file = $request->file('custom404_image_upload');
                        $ext = strtolower(pathinfo($uploaded_file['name'], PATHINFO_EXTENSION));
                        if (!in_array($ext, $allowed_extensions)) {
                            throw new \Exception($this->language->lang('ACP_CUSTOM404_INVALID_FILE_TYPE', implode(', ', array_map('strtoupper', $allowed_extensions))));
                        }
                        $new_image_path = $upload_handler->handle_upload($uploaded_file, $upload_dir, $allowed_extensions, $max_file_size);
                        if ($new_image_path) {
                            $relative_image_path = 'ext/mundophpbb/custom404/uploads/' . basename($new_image_path);
                            $config->set('custom404_image_path', $relative_image_path);
                            $success_message = $this->language->lang('ACP_CUSTOM404_IMAGE_UPLOADED');
                        }
                    } catch (\Exception $e) {
                        $error_message = $e->getMessage();
                    }
                }
            }

            // Manipular submissão do formulário
            if ($request->is_set_post('submit')) {
                if (!check_form_key('mundophpbb_custom404')) {
                    $error_message = $this->language->lang('FORM_INVALID');
                } else {
                    $config->set('custom404_title_key', utf8_normalize_nfc($request->variable('custom404_title_key', 'CUSTOM404_DEFAULT_TITLE', true)));
                    $config->set('custom404_message_key', utf8_normalize_nfc($request->variable('custom404_message_key', 'CUSTOM404_DEFAULT_MESSAGE', true)));
                    $config->set('custom404_image_alt_key', utf8_normalize_nfc($request->variable('custom404_image_alt_key', 'CUSTOM404_DEFAULT_IMAGE_ALT', true)));
                    $config->set('custom404_image_path', $request->variable('custom404_image_path', ''));
                    $config->set('custom404_theme', $request->variable('custom404_theme', 'light')); // Salvar tema selecionado
                    $success_message = $this->language->lang('ACP_CUSTOM404_SAVED');
                }
            }

            // Manipular exportação de logs
            if ($request->is_set_post('export_logs')) {
                $sql = 'SELECT * FROM ' . $this->phpbb_container->getParameter('core.table_prefix') . 'custom404_logs ORDER BY request_time DESC';
                $result = $db->sql_query($sql);

                $csv = fopen('php://output', 'w');
                header('Content-Type: text/csv');
                header('Content-Disposition: attachment; filename="custom404_logs.csv"');
                fputcsv($csv, ['URL', 'IP Address', 'Request Time', 'Referrer', 'User Agent']);

                while ($row = $db->sql_fetchrow($result)) {
                    fputcsv($csv, [
                        $row['request_url'],
                        $row['client_ip'],
                        $this->user->format_date($row['request_time']),
                        $row['referrer'],
                        $row['user_agent'],
                    ]);
                }

                $db->sql_freeresult($result);
                fclose($csv);
                exit;
            }

            // Manipular exclusão de logs
            if ($request->is_set_post('delete_logs')) {
                if (!check_form_key('mundophpbb_custom404')) {
                    $error_message = $this->language->lang('FORM_INVALID');
                } else {
                    $sql = 'DELETE FROM ' . $this->phpbb_container->getParameter('core.table_prefix') . 'custom404_logs';
                    $db->sql_query($sql);
                    $success_message = $this->language->lang('ACP_CUSTOM404_LOGS_DELETED');
                }
            }

            // Recuperar logs com limite
            $log_limit = $request->variable('log_limit', 10, false, \phpbb\request\request_interface::POST);
            $log_limit = in_array($log_limit, [10, 25, 50, 100]) ? $log_limit : 10;
            $sql = 'SELECT * FROM ' . $this->phpbb_container->getParameter('core.table_prefix') . 'custom404_logs ORDER BY request_time DESC LIMIT ' . (int)$log_limit;
            $result = $db->sql_query($sql);
            while ($row = $db->sql_fetchrow($result)) {
                $template->assign_block_vars('logs', [
                    'REQUEST_URL' => $row['request_url'],
                    'CLIENT_IP' => $row['client_ip'],
                    'REQUEST_TIME' => $this->user->format_date($row['request_time']),
                    'REFERRER' => $row['referrer'],
                    'USER_AGENT' => $row['user_agent'],
                ]);
            }
            $db->sql_freeresult($result);

            // Preparar mensagem para edição
            $message_key = $config['custom404_message_key'] ?? 'CUSTOM404_DEFAULT_MESSAGE';
            $message = $this->language->lang($message_key);
            $uid = $bitfield = $flags = '';
            generate_text_for_storage($message, $uid, $bitfield, $flags, true, true, true);

            // Verificar disponibilidade do SCEditor
            $core_path = $this->phpbb_container->getParameter('core.root_path');
            $sceditor_available = file_exists($core_path . 'adm/style/posting_buttons.html') && file_exists($core_path . 'adm/style/posting_editor.html');

            // Carregar imagens disponíveis
            $images = [];
            if ($this->filesystem && $this->filesystem->exists($upload_dir)) {
                $files = scandir($upload_dir);
                foreach ($files as $file) {
                    $ext = strtolower(pathinfo($file, PATHINFO_EXTENSION));
                    if (in_array($ext, $allowed_extensions) && $this->filesystem->is_readable($upload_dir . $file)) {
                        $images[] = $file;
                    }
                }
                sort($images);
            }

            $image_options = [];
            $delete_image_options = [];
            $default_image = '404_error.png';
            $default_relative_path = 'ext/mundophpbb/custom404/uploads/' . $default_image;
            $fallback_image_path = 'styles/prosilver/theme/images/no_image.png';
            $default_absolute_path = $this->filesystem && $this->filesystem->exists($upload_dir . $default_image)
                ? generate_board_url() . '/' . $default_relative_path
                : generate_board_url() . '/' . $fallback_image_path;

            $image_options[] = [
                'VALUE' => $default_relative_path,
                'SELECTED' => (empty($config['custom404_image_path']) || $config['custom404_image_path'] === $default_relative_path) ? ' selected="selected"' : '',
                'LABEL' => $this->language->lang('ACP_CUSTOM404_DEFAULT_IMAGE'),
                'ABSOLUTE_PATH' => $default_absolute_path
            ];

            foreach ($images as $image) {
                if ($image !== $default_image) {
                    $encoded_image = rawurlencode($image);
                    $relative_image_path = 'ext/mundophpbb/custom404/uploads/' . $encoded_image;
                    $absolute_image_path = generate_board_url() . '/' . $relative_image_path;
                    if ($this->filesystem && $this->filesystem->exists($upload_dir . $image)) {
                        $image_options[] = [
                            'VALUE' => $relative_image_path,
                            'SELECTED' => ($config['custom404_image_path'] === $relative_image_path) ? ' selected="selected"' : '',
                            'LABEL' => htmlspecialchars_decode($image),
                            'ABSOLUTE_PATH' => $absolute_image_path
                        ];
                        $delete_image_options[] = [
                            'VALUE' => $relative_image_path,
                            'LABEL' => htmlspecialchars_decode($image)
                        ];
                    }
                }
            }

            $selected_image_path = $config['custom404_image_path'] && $this->filesystem && $this->filesystem->exists($this->phpbb_container->getParameter('core.root_path') . $config['custom404_image_path'])
                ? generate_board_url() . '/' . $config['custom404_image_path']
                : $default_absolute_path;

            // Formatar extensões permitidas e tamanho máximo do arquivo
            $extensions_string = !empty($allowed_extensions) ? implode(', ', array_map('strtoupper', $allowed_extensions)) : 'N/A';
            $max_size_mb = (int) ($max_file_size / 1024 / 1024);

            // Atribuir variáveis ao template
            $template->assign_vars([
                'U_ACTION' => $this->u_action,
                'CUSTOM404_TITLE' => $this->language->lang($config['custom404_title_key'] ?? 'CUSTOM404_DEFAULT_TITLE'),
                'CUSTOM404_MESSAGE' => generate_text_for_edit($message, $uid, $flags)['text'],
                'CUSTOM404_IMAGE_ALT' => $this->language->lang($config['custom404_image_alt_key'] ?? 'CUSTOM404_DEFAULT_IMAGE_ALT'),
                'CUSTOM404_IMAGE_PATH' => $selected_image_path,
                'DEFAULT_IMAGE_PATH' => $default_absolute_path,
                'CUSTOM404_THEME' => $config['custom404_theme'] ?? 'light',
                'S_BBCODE_ALLOWED' => true,
                'S_BBCODE_IMG' => true,
                'S_BBCODE_URL' => true,
                'S_BBCODE_QUOTE' => true,
                'S_BBCODE_FLASH' => false,
                'S_FORM_ENCODE' => ' enctype="multipart/form-data"',
                'S_BBCODE_EDITOR' => $sceditor_available,
                'SIG_EDIT' => false,
                'S_LINKS_ALLOWED' => true,
                'MAX_FONT_SIZE' => $config['max_post_font_size'] ?? 200,
                'ALLOWED_EXTENSIONS' => $extensions_string,
                'MAX_FILE_SIZE_MB' => $max_size_mb,
                'L_ACP_CUSTOM404_UPLOAD_INFO' => $this->language->lang('ACP_CUSTOM404_UPLOAD_INFO', $extensions_string, $max_size_mb),
                'L_ACP_CUSTOM404_UPLOAD_IMAGE' => $this->language->lang('ACP_CUSTOM404_UPLOAD_IMAGE'),
                'L_ACP_CUSTOM404_SAVE_CHANGES' => $this->language->lang('ACP_CUSTOM404_SAVE_CHANGES'),
                'L_ACP_CUSTOM404_DELETE' => $this->language->lang('ACP_CUSTOM404_DELETE'),
                'L_ACP_CUSTOM404_IMAGE_PATH_LABEL' => $this->language->lang('ACP_CUSTOM404_IMAGE_PATH_LABEL'),
                'L_ACP_CUSTOM404_CHOOSE_FILE' => $this->language->lang('ACP_CUSTOM404_CHOOSE_FILE'),
                'L_ACP_CUSTOM404_NO_FILE_CHOSEN' => $this->language->lang('ACP_CUSTOM404_NO_FILE_CHOSEN'),
                'L_ACP_CUSTOM404_THEME_LABEL' => $this->language->lang('ACP_CUSTOM404_THEME_LABEL'),
                'S_SUCCESS_MESSAGE' => $success_message,
                'S_ERROR_MESSAGE' => $error_message,
                'LOG_LIMIT' => $log_limit,
            ]);

            // Atribuir opções de limite de logs
            $log_limit_options = [
                10 => '10',
                25 => '25',
                50 => '50',
                100 => '100',
            ];
            foreach ($log_limit_options as $value => $label) {
                $template->assign_block_vars('log_limit_options', [
                    'VALUE' => $value,
                    'LABEL' => $label,
                    'SELECTED' => ($value == $log_limit) ? ' selected="selected"' : '',
                ]);
            }

            // Atribuir opções de tema
            $theme_options = [
                'light' => $this->language->lang('ACP_CUSTOM404_THEME_LIGHT'),
                'dark' => $this->language->lang('ACP_CUSTOM404_THEME_DARK'),
            ];
            foreach ($theme_options as $value => $label) {
                $template->assign_block_vars('theme_options', [
                    'VALUE' => $value,
                    'LABEL' => $label,
                    'SELECTED' => ($value == ($config['custom404_theme'] ?? 'light')) ? ' checked="checked"' : '',
                ]);
            }

            foreach ($image_options as $option) {
                $template->assign_block_vars('image_options', $option);
            }

            foreach ($delete_image_options as $option) {
                $template->assign_block_vars('delete_image_options', $option);
            }

            if ($sceditor_available) {
                $this->user->add_lang('posting');
                include_once($core_path . 'includes/functions_posting.' . $this->phpbb_container->getParameter('core.php_ext'));
            }
        } catch (\Exception $e) {
            error_log('Custom 404 Error: ' . $e->getMessage() . ' in ' . $e->getFile() . ' on line ' . $e->getLine() . "\n" . $e->getTraceAsString(), 3, $log_file);
            $template->assign_vars([
                'S_ERROR_MESSAGE' => 'Ocorreu um erro: ' . htmlspecialchars($e->getMessage()),
            ]);
        }
    }

    /**
     * Carregar arquivos de idioma para o ACP
     */
    protected function load_language_files()
    {
        $lang_path = $this->phpbb_container && method_exists($this->phpbb_container, 'getParameter')
            ? $this->phpbb_container->getParameter('core.root_path') . 'ext/mundophpbb/custom404/language/'
            : dirname(__FILE__) . '/../../language/';
        $log_dir = $this->phpbb_container && method_exists($this->phpbb_container, 'getParameter')
            ? $this->phpbb_container->getParameter('core.root_path') . 'ext/mundophpbb/custom404/logs/'
            : dirname(__FILE__) . '/../../logs/';
        $user_lang = $this->user ? $this->user->lang_name : 'en';
        $fallback_lang = 'en';
        $primary_lang_file = $lang_path . $user_lang . '/acp_common.php';

        if ($this->filesystem && !$this->filesystem->exists($log_dir)) {
            $this->filesystem->mkdir($log_dir, 0755);
        }
        error_log('Custom 404 Language: ' . $user_lang . ' | File: ' . $primary_lang_file, 3, $log_dir . 'debug.log');

        if ($this->filesystem && $this->filesystem->exists($primary_lang_file)) {
            $this->language->add_lang(['acp_common', 'common'], 'mundophpbb/custom404', $user_lang);
        } else {
            $this->language->add_lang(['acp_common', 'common'], 'mundophpbb/custom404', $fallback_lang);
        }

        $this->language->add_lang(['common', 'acp/common']);
    }
}