/**
 * JavaScript for the Custom 404 ACP
 */
document.addEventListener('DOMContentLoaded', () => {
    // Form validation
    const form = document.querySelector('form[action="{U_ACTION}"]');
    const submitButton = document.querySelector('.submit-buttons input[name="submit"]');

    if (form && submitButton) {
        form.addEventListener('submit', (e) => {
            const titleInput = document.getElementById('custom404_title_key');
            const messageInput = document.getElementById('custom404_message_key');
            let errors = [];

            if (!titleInput.value.trim()) {
                errors.push('{L_ACP_CUSTOM404_TITLE_REQUIRED}');
            }

            if (!messageInput.value.trim()) {
                errors.push('{L_ACP_CUSTOM404_MESSAGE_REQUIRED}');
            }

            if (errors.length > 0) {
                e.preventDefault();
                alert(errors.join('\n'));
            }
        });
    }

    // Update image preview on page load
    updateImagePreview();

    // Handle file input change
    const fileInput = document.getElementById('custom404_image_upload');
    if (fileInput) {
        fileInput.addEventListener('change', function() {
            const fileName = this.files.length > 0 ? this.files[0].name : '{L_ACP_CUSTOM404_NO_FILE_CHOSEN}';
            document.getElementById('file-name').textContent = fileName;
        });
    }

    // Attach event listener for image preview
    const imageSelect = document.getElementById('custom404_image_path');
    if (imageSelect) {
        imageSelect.addEventListener('change', updateImagePreview);
    }
});

/**
 * Insert BBCode tags into the textarea
 *
 * @param {string} startTag The opening BBCode tag
 * @param {string} endTag The closing BBCode tag
 */
function insertBBCode(startTag, endTag) {
    var textarea = document.getElementById("custom404_message_key");
    var start = textarea.selectionStart;
    var end = textarea.selectionEnd;
    var text = textarea.value;
    var selectedText = text.substring(start, end);
    var newText = text.substring(0, start) + startTag + selectedText + endTag + text.substring(end);
    textarea.value = newText;
    textarea.focus();
}

/**
 * Update the image preview based on the selected option
 */
function updateImagePreview() {
    var select = document.getElementById('custom404_image_path');
    var preview = document.getElementById('image_preview');
    var message = document.getElementById('preview_message');
    var option = select.options[select.selectedIndex];
    var path = option.getAttribute('data-absolute-path') || '{DEFAULT_IMAGE_PATH}';
    
    // Get translations from a hidden element or fallback to template
    var translations = document.getElementById('custom404-translations');
    var imagePreviewText = translations ? translations.dataset.imagePreview : '{L_ACP_CUSTOM404_IMAGE_PREVIEW}';
    var imageNotFoundText = translations ? translations.dataset.imageNotFound : '{L_ACP_CUSTOM404_IMAGE_NOT_FOUND}';

    if (path && path !== '{DEFAULT_IMAGE_PATH}') {
        preview.src = path;
        preview.style.display = 'block';
        message.textContent = imagePreviewText;
    } else {
        preview.src = '{DEFAULT_IMAGE_PATH}';
        preview.style.display = 'block';
        message.textContent = imageNotFoundText;
    }
}